@@ -32,7 +32,9 @@
 
 #include <libtracker-common/tracker-common.h>
 
+#if HAVE_TRACKER_FTS
 #include <libtracker-fts/tracker-fts.h>
+#endif
 
 #include <libtracker-db/tracker-db-interface-sqlite.h>
 #include <libtracker-db/tracker-db-manager.h>
@@ -741,7 +743,7 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 	gint i;
 
 	/* This updates property-property changes and marks classes for necessity
-	 * of having their tables recreated later. There's support for 
+	 * of having their tables recreated later. There's support for
 	 * tracker:notify, tracker:writeback and tracker:indexed */
 
 	if (seen_classes) {
@@ -833,7 +835,7 @@ tracker_data_ontology_process_changes (GPtrArray *seen_classes,
 			}
 
 			if (update_property_value ("rdfs:range", subject, RDFS_PREFIX "range",
-			                           tracker_class_get_uri (tracker_property_get_range (property)), 
+			                           tracker_class_get_uri (tracker_property_get_range (property)),
 			                           allowed_range_conversions,
 			                           NULL, property)) {
 				TrackerClass *class;
@@ -1215,7 +1217,7 @@ class_add_super_classes_from_db (TrackerDBInterface *iface,
 			TrackerClass *super_class;
 			const gchar *super_class_uri;
 
-			super_class_uri = tracker_db_cursor_get_string (cursor, 0);
+			super_class_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 			super_class = tracker_ontologies_get_class_by_uri (super_class_uri);
 			tracker_class_add_super_class (class, super_class);
 		}
@@ -1252,7 +1254,7 @@ property_add_super_properties_from_db (TrackerDBInterface *iface,
 			TrackerProperty *super_property;
 			const gchar *super_property_uri;
 
-			super_property_uri = tracker_db_cursor_get_string (cursor, 0);
+			super_property_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 			super_property = tracker_ontologies_get_property_by_uri (super_property_uri);
 			tracker_property_add_super_property (property, super_property);
 		}
@@ -1287,7 +1289,7 @@ db_get_static_data (TrackerDBInterface *iface)
 
 			ontology = tracker_ontology_new ();
 
-			uri = tracker_db_cursor_get_string (cursor, 0);
+			uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 			last_mod = (time_t) tracker_db_cursor_get_int (cursor, 1);
 
 			tracker_ontology_set_is_new (ontology, FALSE);
@@ -1324,8 +1326,8 @@ db_get_static_data (TrackerDBInterface *iface)
 
 			namespace = tracker_namespace_new ();
 
-			uri = tracker_db_cursor_get_string (cursor, 0);
-			prefix = tracker_db_cursor_get_string (cursor, 1);
+			uri = tracker_db_cursor_get_string (cursor, 0, NULL);
+			prefix = tracker_db_cursor_get_string (cursor, 1, NULL);
 
 			tracker_namespace_set_is_new (namespace, FALSE);
 			tracker_namespace_set_uri (namespace, uri);
@@ -1368,12 +1370,12 @@ db_get_static_data (TrackerDBInterface *iface)
 			class = tracker_class_new ();
 
 			id = tracker_db_cursor_get_int (cursor, 0);
-			uri = tracker_db_cursor_get_string (cursor, 1);
+			uri = tracker_db_cursor_get_string (cursor, 1, NULL);
 
 			tracker_db_cursor_get_value (cursor, 2, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				notify = (g_value_get_int (&value) == 1);
+				notify = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1455,14 +1457,14 @@ db_get_static_data (TrackerDBInterface *iface)
 			property = tracker_property_new ();
 
 			id = tracker_db_cursor_get_int (cursor, 0);
-			uri = tracker_db_cursor_get_string (cursor, 1);
-			domain_uri = tracker_db_cursor_get_string (cursor, 2);
-			range_uri = tracker_db_cursor_get_string (cursor, 3);
+			uri = tracker_db_cursor_get_string (cursor, 1, NULL);
+			domain_uri = tracker_db_cursor_get_string (cursor, 2, NULL);
+			range_uri = tracker_db_cursor_get_string (cursor, 3, NULL);
 
 			tracker_db_cursor_get_value (cursor, 4, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				multi_valued = (g_value_get_int (&value) > 1);
+				multi_valued = (g_value_get_int64 (&value) > 1);
 				g_value_unset (&value);
 			} else {
 				/* nrl:maxCardinality not set
@@ -1473,19 +1475,19 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_db_cursor_get_value (cursor, 5, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				indexed = (g_value_get_int (&value) == 1);
+				indexed = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
 				indexed = FALSE;
 			}
 
-			secondary_index_uri = tracker_db_cursor_get_string (cursor, 6);
+			secondary_index_uri = tracker_db_cursor_get_string (cursor, 6, NULL);
 
 			tracker_db_cursor_get_value (cursor, 7, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				fulltext_indexed = (g_value_get_int (&value) == 1);
+				fulltext_indexed = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1495,7 +1497,7 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_db_cursor_get_value (cursor, 8, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				fulltext_no_limit = (g_value_get_int (&value) == 1);
+				fulltext_no_limit = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1505,7 +1507,7 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_db_cursor_get_value (cursor, 9, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				transient = (g_value_get_int (&value) == 1);
+				transient = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1515,7 +1517,7 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_db_cursor_get_value (cursor, 10, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				annotation = (g_value_get_int (&value) == 1);
+				annotation = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1527,7 +1529,7 @@ db_get_static_data (TrackerDBInterface *iface)
 			tracker_db_cursor_get_value (cursor, 11, &value);
 
 			if (G_VALUE_TYPE (&value) != 0) {
-				writeback = (g_value_get_int (&value) == 1);
+				writeback = (g_value_get_int64 (&value) == 1);
 				g_value_unset (&value);
 			} else {
 				/* NULL */
@@ -1545,7 +1547,7 @@ db_get_static_data (TrackerDBInterface *iface)
 				is_inverse_functional_property = FALSE;
 			}
 
-			default_value = tracker_db_cursor_get_string (cursor, 13);
+			default_value = tracker_db_cursor_get_string (cursor, 13, NULL);
 
 			tracker_property_set_is_new (property, FALSE);
 			tracker_property_set_transient (property, transient);
@@ -1913,9 +1915,9 @@ create_decomposed_metadata_tables (TrackerDBInterface *iface,
 
 				if (in_update) {
 					g_debug ("%sAltering database for class '%s' property '%s': single value (%s)",
-					         in_alter ? "" : "  ", 
-					         service_name, 
-					         field_name, 
+					         in_alter ? "" : "  ",
+					         service_name,
+					         field_name,
 					         in_alter ? "alter" : "create");
 				}
 
@@ -2110,7 +2112,7 @@ create_decomposed_transient_metadata_tables (TrackerDBInterface *iface)
 	}
 }
 
-void 
+void
 tracker_data_ontology_import_finished (void)
 {
 	TrackerClass **classes;
@@ -2277,7 +2279,9 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		return FALSE;
 	}
 
+#if HAVE_TRACKER_FTS
 	tracker_fts_set_map_function (tracker_ontologies_get_uri_by_id);
+#endif
 
 	if (first_time != NULL) {
 		*first_time = is_first_time_index;
@@ -2319,14 +2323,17 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 		/* Read first ontology and commit it into the DB */
 		tracker_data_begin_db_transaction_for_replay (tracker_db_journal_reader_get_time ());
-		tracker_db_interface_sqlite_fts_init (TRACKER_DB_INTERFACE_SQLITE (iface), TRUE);
+
+		/* This is a no-op when FTS is disabled */
+		tracker_db_interface_sqlite_fts_init (iface, TRUE);
+
 		tracker_data_ontology_import_into_db (FALSE);
 		tracker_data_commit_db_transaction ();
 		tracker_db_journal_reader_shutdown ();
 
 		/* Start replay. Ontology changes might happen during replay of the journal. */
 
-		tracker_data_replay_journal (classes, properties, id_uri_map, 
+		tracker_data_replay_journal (classes, properties, id_uri_map,
 		                             busy_callback, busy_user_data, busy_status);
 
 		in_journal_replay = FALSE;
@@ -2375,7 +2382,9 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		/* Not an ontology transaction: this is the first ontology */
 		tracker_db_journal_start_transaction (time (NULL));
 
-		tracker_db_interface_sqlite_fts_init (TRACKER_DB_INTERFACE_SQLITE (iface), TRUE);
+		/* This is a no-op when FTS is disabled */
+		tracker_db_interface_sqlite_fts_init (iface, TRUE);
+
 		tracker_data_ontology_import_into_db (FALSE);
 
 		/* store ontology in database */
@@ -2413,7 +2422,8 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 		create_decomposed_transient_metadata_tables (iface);
 		check_ontology = TRUE;
 
-		tracker_db_interface_sqlite_fts_init (TRACKER_DB_INTERFACE_SQLITE (iface), FALSE);
+		/* This is a no-op when FTS is disabled */
+		tracker_db_interface_sqlite_fts_init (iface, FALSE);
 	}
 
 	if (check_ontology) {
@@ -2480,7 +2490,7 @@ tracker_data_manager_init (TrackerDBManagerFlags  flags,
 
 		if (cursor) {
 			while (tracker_db_cursor_iter_next (cursor, &error)) {
-				const gchar *onto_uri = tracker_db_cursor_get_string (cursor, 0);
+				const gchar *onto_uri = tracker_db_cursor_get_string (cursor, 0, NULL);
 				/* It's stored as an int in the db anyway. This is caused by
 				 * string_to_gvalue in tracker-data-update.c */
 				gint value = tracker_db_cursor_get_int (cursor, 1);
