@@ -30,17 +30,16 @@
 typedef struct {
 	GHashTable *allowances;
 	GArray *events;
+	GStringChunk *chunk;
 } EventsPrivate;
 
-static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
+static EventsPrivate *private;
 
 static void
 tracker_events_add_allow (const gchar *rdf_class)
 {
-	EventsPrivate *private;
 	TrackerClass *cl;
 
-	private = g_static_private_get (&private_key);
 	g_return_if_fail (private != NULL);
 
 	cl = tracker_ontologies_get_class_by_uri (rdf_class);
@@ -72,10 +71,14 @@ prepare_event_for_rdf_type (EventsPrivate *private,
 		private->events = g_array_new (TRUE, FALSE, sizeof (TrackerEvent));
 	}
 
+	if (!private->chunk) {
+		private->chunk = g_string_chunk_new (strlen (uri) + 10);
+	}
+
 	event.type = type;
 	event.class = rdf_class;
 	event.predicate = tracker_ontologies_get_property_by_uri (predicate);
-	event.subject = g_strdup (uri);
+	event.subject = g_string_chunk_insert_const (private->chunk, uri);
 
 	g_array_append_val (private->events, event);
 }
@@ -87,10 +90,7 @@ tracker_events_insert (const gchar *uri,
                        GPtrArray *rdf_types,
                        TrackerDBusEventsType type)
 {
-	EventsPrivate *private;
-
 	g_return_if_fail (rdf_types || type != TRACKER_DBUS_EVENTS_TYPE_UPDATE);
-	private = g_static_private_get (&private_key);
 	g_return_if_fail (private != NULL);
 
 	if (rdf_types && type == TRACKER_DBUS_EVENTS_TYPE_UPDATE) {
@@ -116,27 +116,15 @@ tracker_events_insert (const gchar *uri,
 	}
 }
 
-static void
-tracker_event_destroy (TrackerEvent *event)
-{
-	g_free (event->subject);
-}
-
 void
 tracker_events_reset (void)
 {
-	EventsPrivate *private;
-	guint i;
-
-	private = g_static_private_get (&private_key);
 	g_return_if_fail (private != NULL);
 
 	if (private->events) {
-		for (i = 0; i < private->events->len; i++) {
-			tracker_event_destroy (&g_array_index (private->events, TrackerEvent, i));
-		}
 		g_array_free (private->events, TRUE);
-
+		g_string_chunk_free (private->chunk);
+		private->chunk = NULL;
 		private->events = NULL;
 	}
 }
@@ -144,9 +132,6 @@ tracker_events_reset (void)
 GArray *
 tracker_events_get_pending (void)
 {
-	EventsPrivate *private;
-
-	private = g_static_private_get (&private_key);
 	g_return_val_if_fail (private != NULL, NULL);
 
 	return private->events;
@@ -162,16 +147,11 @@ free_private (EventsPrivate *private)
 void
 tracker_events_init (TrackerNotifyClassGetter callback)
 {
-	EventsPrivate *private;
 	GStrv          classes_to_signal;
 	gint           i, count;
 
 	private = g_new0 (EventsPrivate, 1);
 
-	g_static_private_set (&private_key,
-	                      private,
-	                      (GDestroyNotify) free_private);
-
 	private->allowances = g_hash_table_new (g_direct_hash, g_direct_equal);
 
 	private->events = NULL;
@@ -196,12 +176,10 @@ tracker_events_init (TrackerNotifyClassGetter callback)
 void
 tracker_events_shutdown (void)
 {
-	EventsPrivate *private;
-
-	private = g_static_private_get (&private_key);
 	if (private != NULL) {
 		tracker_events_reset ();
-		g_static_private_set (&private_key, NULL, NULL);
+		free_private (private);
+		private = NULL;
 	} else {
 		g_warning ("tracker_events already shutdown");
 	}
